﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode {
  [StorableClass]
  public class Tank : CodeNode {
    public override int MinimumArity { get { return 7; } }
    public override int MaximumArity { get { return 7; } }

    [Storable]
    public override string Prefix { get; set; }

    [Storable]
    public override string Suffix { get; set; }

    [StorableConstructor]
    private Tank(bool deserializing) : base(deserializing) { }
    private Tank(Tank original, Cloner cloner)
      : base(original, cloner) {
    }

    public Tank()
      : base("Tank", "The root of a Robocode Tank program.") {
      this.Prefix = "package Evaluation;" +
                    "\r\nimport robocode.*;" +
                    "\r\nimport robocode.Robot;" +
                    "\r\nimport robocode.util.*;" +
                    "\r\nimport static robocode.util.Utils.normalRelativeAngleDegrees;" +
                    "\r\nimport java.awt.*;" +
                    "\r\n\r\n" +
                    "\r\npublic class output extends AdvancedRobot {\r\n";
      this.Suffix = "\r\n}";
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Tank(this, cloner);
    }

    public override string Interpret(ISymbolicExpressionTreeNode node, System.Collections.Generic.IEnumerable<ISymbolicExpressionTreeNode> children) {
      string result = "";
      foreach (ISymbolicExpressionTreeNode c in children)
        result += "\r\n" + ((CodeNode)c.Symbol).Interpret(c, c.Subtrees);
      return this.Prefix + "\r\n" + result + "\r\n" + this.Suffix;
    }
  }
}